-- fsshell, File access in LUA (v1.01)
-- Scripts must always begin with comment!

function event (e)
	-- Print system events
	if e.type < ALARM_TYPE_INTERNAL then
    os.trace("Event %d/%d ... (%d ms)", e.type, e.idx, e.time % 60000)
	end
end

function justWords(str)
	local t = {}
	
	local function helper(word)
		table.insert(t, word)
		return "";
	end
	
	if (str == nil) then 
	  str = "" 
	end
  str:gsub("%S+", helper)
  return t;
end

function help ()
	print("AVL file access (v1.1 @28/07/2020)");
	print("");
	print("Using:");
	print("dir \"dir\"");
	print("mkdir \"dir\"");
	print("rmdir \"dir\"");
	print("stat \"file.txt\"");
	print("read \"file.txt\"");
	print("dump \"file.txt\"");
	print("remove \"file.txt\"");
	print("rename \"from.txt\" \"to.txt\"");
	print("copy \"from.txt\" \"to.txt\"");
	print("bench size \"to.txt\"");
	print("mount SD");
	print("unmount SD");
	print("info SD|LFS");
	print("exit");
	print("exec \"script.lua\"");
	print("");
end

function file_stat(tmp, st)
  if (st) then
		local s = st["mode"]; 
	  
	  print(string.format("%c%-6s '%s'", s:byte(1)-32, string.sub(s,2), tmp));

		s = st["nlink"]; if s then print("nlink  ",s) end
		s = st["uid"]; if s then print("uid    ",s) end
		s = st["gid"]; if s then print("gid    ",s) end
		s = st["access"]; if s then print("access ",s) end
		s = st["size"]; if s then print("size   ",s) end
		s = st["permissions"]; if s then print("permission",s) end
		
  else
	  print(string.format("Not found '%s'",tmp));
  end	
end

function dir_stat(tmp)
	local result = {}
	local count = 0
  local file;
  
	--[[
  local iter, dir = os.dir(tmp)
	
	if (dir) then
	  file = dir:next()
	
	  while file do
	    if (file == "." or file == "..") then
	   		table.insert(result, file);
	    else
		    local st = os.stat(tmp.."/"..file)
		    if (st["mode"] == "directory") then
		   		table.insert(result, string.format("%s  %s", file, st["permissions"], st["size"]));
		    else
		   		table.insert(result, string.format("  %s  %s %s", file, st["permissions"], st["size"]));
		    end
	    end
	    count = count + 1
	    file = dir:next()
	  end
	end
  --]]
		
	for file in os.dir(tmp) do
	  if (file == "." or file == "..") then
	 		table.insert(result, file);
	  else
	   local st = os.stat(tmp.."/"..file)
	   if (st["mode"] == "directory") then
	  		table.insert(result, string.format("%s  %s %s", 
	  										file, st["permissions"], os.date("%d/%b/%y %H:%M:%S", st["access"]), st["size"]));
	   else
	  		table.insert(result, string.format("  %s  %s %s %s", 
	  										file, st["permissions"], os.date("%d/%b/%y %H:%M:%S", st["access"]), st["size"]));
	    end
	  end
	  count = count + 1
	end

	if (count > 0) then
		print(string.format("Content of '%s'", tmp));
	
		for i,v in ipairs(result) do
			print(v)
		end	
	
		print(string.format("%d entries",count));
	else
		print(string.format("Directory '%s' not exist", tmp));
	end
end

function hexdump(offset, buf)
  local line = ""
  for i=1, math.ceil(#buf/32) * 32 do
     if (i-1) % 32 == 0 then 
       line = string.format('%06X  ', offset + i-1) 
     end
     line = line .. (i > #buf and '   ' or string.format('%02X ', buf:byte(i)))
     if i % 8 == 0 then line = line .. ' ' end
     if i % 32 == 0 then print(line .. '|  ' .. buf:sub(i-32+1, i):gsub('%c','.')) end
  end
end

-- Use shell commands with i.e. $PFAL,SYS.Lua.event,0,"help"

local commands = {}

commands["stat"] = function(arg1, arg2) 
	if (arg1 or help()) then
		file_stat(arg1, os.stat(arg1));
	end	
end

commands["dir"] = function(arg1, arg2) 
	if (arg1 or help()) then
		dir_stat(arg1);
	end	
end

commands["rename"] = function(arg1, arg2) 
	if (arg1 or help()) then
		if os.rename(arg1, arg2) == nil then 
			print(string.format("Can't rename file or directory '%s'", arg1)) 
		else
			print(string.format("Rename file or directory '%s'", arg1)) 
		end
	end	
end

commands["remove"] = function(arg1, arg2) 
	if (arg1 or help()) then
		if os.remove(arg1) == nil then 
			print(string.format("Can't remove file or directory '%s'", arg1)) 
		else
			print(string.format("Remove file or directory '%s'", arg1)) 
		end
	end	
end

commands["mkdir"] = function(arg1, arg2) 
	if (arg1 or help()) then
		if os.mkdir(arg1) == nil then 
			print(string.format("Can't make directory '%s'", arg1)) 
		else
			print(string.format("Create directory '%s'", arg1)) 
		end
	end	
end

commands["rmdir"] = function(arg1, arg2) 
	if (arg1 or help()) then
		if os.rmdir(arg1) == nil then 
			print(string.format("Can't remove directory '%s'", arg1)) 
		else
			print(string.format("Remove directory '%s'", arg1)) 
		end
	end	
end

commands["read"] = function(arg1, arg2) 
	if (arg1 or help()) then
		local bytes = os.stat(arg1, "size")
		local file = io.open(arg1);
		if (file) then
		  for line in file:lines() do print(line) end
		  file:close()
		  print(string.format("file '%s' (%d bytes)", arg1, bytes))
		else
	    print(string.format("Not found '%s'",arg1));
		end  
	end	
end

commands["dump"] = function(arg1, arg2) 
	if (arg1 or help()) then
		local bytes = os.stat(arg1, "size")
		local file = io.open(arg1);
		if (file) then
		  local n, line = 0, file:read(1024)
		  while line do 
		  	hexdump(n, line); 
		  	n = n + 1024; 
		  	line = file:read(1024)  
	  	end
		  file:close()
		  print(string.format("file '%s' (%d bytes)", arg1, bytes))
		else
	    print(string.format("Not found '%s'",arg1));
		end  
	end	
end

commands["copy"] = function(arg1, arg2) 
	if (arg1 or help()) then
	if (arg2 or help()) then
		local file1 = io.open(arg1, "r");
		local file2 = io.open(arg2, "w");
		if (file1 and file2) then
		  local t = os.clock()
		  local line = file1:read(4096)
		  while line do 
		  	file2:write(line)
		  	line = file1:read(4096)  
	  	end
		  file1:close()
		  file2:close()
		  t = os.clock() - t;
		  print(string.format("write '%s' in %.2f seconds (%d bytes)", arg2, t / 1000, os.stat(arg2, "size")))
		else
	    print(string.format("Couldn't write '%s'",arg2));
		end  
	end	
	end	
end

commands["mount"] = function(arg1, arg2) 
	
	if (arg1 or help()) then
		local res, str = avl.pfal("File.SD.Mount")
		if res then print("the sdcard is mounted") end 
	end	
end

commands["unmount"] = function(arg1, arg2) 
	if (arg1 or help()) then
		local res, str = avl.pfal("File.SD.Unmount")
		if res then print("the sdcard is unmounted") end 
	end	
end

commands["bench"] = function(arg1, arg2) 
  local size = tonumber(arg1 or 64000)

	if (arg2 or help()) then
	if (size > 0 or help()) then
		local t, file = 0, io.open(arg2, "w+");
		if (file) then
			local msg = string.rep("123456789_123456789_123456789_123456789_123456789\n", size/50);
			
			-- bench write strategy
		  t = os.clock()
	  	file:write(msg)
		  file:close()
		  t = os.clock() - t
		  size = os.stat(arg2, "size")
		  print(string.format("write %d bytes to '%s' (%.3f Kbyte/s)", size, arg2, (size / 1.024 / t)))

			-- bench read strategy
		  t = os.clock()
		  file = io.open(arg2, "r");
		  local line = file:read(4096)
		  while line do 
		  	line = file:read(4096)  
	  	end
		  file:close()
		  t = os.clock() - t
		  size = os.stat(arg2, "size")
		  print(string.format("read %d bytes from '%s' (%.3f Kbyte/s)", size, arg2, (size / 1.024 / t)))

		else
	    print(string.format("Couldn't write '%s'",arg2));
		end  
	end
	end
end

commands["info"] = function(arg1, arg2) 
	if (arg1 or help()) then
		local st = os.drive(arg1)
		print(string.format("%s has %d/%d/%d free/used/total Kbytes",
									string.upper(arg1), st["free"] * st["blocksize"] / 1024,	
									st["used"] * st["blocksize"] / 1024,	
									st["total"] * st["blocksize"] / 1024));	
	end
end

commands["exit"] = function(arg1, arg2) 
	os.exit(arg1)
end

commands["exec"] = function(arg1, arg2) 
  local script = ""
	local l, f;
  
	if (arg1 or help()) then
		f = io.open(arg1)
		if (f) then
			l = f:lines()
			script = l()
			f:close()
		end
		
		if (string.sub(script,1,2) == "--") then
			print(string.format("LUA: %s", script)) 
			print(string.format("Starting script '%s'", arg1)) 
			avl.pfal(string.format("Sys.LUA.Stop"))
			avl.pfal(string.format("Sys.LUA.Start=\"%s\"", arg1))
		else
			print(string.format("Can't start script '%s'", arg1)) 
		end
	end
end

-- Startup notification, print version
os.trace("Running %s on %s, %s", _VERSION, avl.device(), avl.version());

-- Mainloop of the script
while 1 do
  local ev = avl.event(10000)
		
	if (ev) then
  	event (ev)
  	
	  if (ev.type == ALARM_SYS_SERIALDATA0) then
			local t = justWords(ev.u_recvdata);
			local cmd = string.lower(t[1]);

			if (commands[cmd]) then
				commands[cmd](t[2], t[3])
			else
			  help()
			end
		end
		
	  if (ev.type == ALARM_TYPE_INTERNAL) then
			local t = justWords(ev.u_string);
			local cmd = string.lower(t[1]);
			
			if (commands[cmd]) then
				commands[cmd](t[2], t[3])
			else
			  help()
			end
	  end
	end
end
